-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Model the links between the FN and BN in the UniBoard TR mesh at
--          serial transceiver STD_LOGIC level.
-- Description: See unb1_board_mesh_reorder_bidir.vhd

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE work.unb1_board_pkg.ALL;
USE work.tb_unb1_board_pkg.ALL;


ENTITY unb1_board_mesh_model_sl IS
  GENERIC (
    g_reorder   : BOOLEAN := TRUE
  );
  PORT (
    -- FN to BN
    fn_tx_sl_3arr  : IN  t_unb1_board_mesh_sl_3arr := (OTHERS=>(OTHERS=>(OTHERS=>'0')));  -- _3arr = (node id 3,2,1,0)(bus id 3,2,1,0)(tr lane 3,2,1,0)
    bn_rx_sl_3arr  : OUT t_unb1_board_mesh_sl_3arr;
    
    -- BN to FN
    bn_tx_sl_3arr  : IN  t_unb1_board_mesh_sl_3arr := (OTHERS=>(OTHERS=>(OTHERS=>'0')));
    fn_rx_sl_3arr  : OUT t_unb1_board_mesh_sl_3arr
  );
END unb1_board_mesh_model_sl;


ARCHITECTURE beh OF unb1_board_mesh_model_sl IS
BEGIN

  -- Functional mesh connect for transpose
  no_reorder : IF g_reorder=FALSE GENERATE
    -- BN(i)(j) <= FN(j)(i)
    -- FN(i)(j) <= BN(j)(i)
    gen_i : FOR i IN 0 TO 3 GENERATE
      gen_j : FOR j IN 0 TO 3 GENERATE
        bn_rx_sl_3arr(i)(j) <= fn_tx_sl_3arr(j)(i);
        fn_rx_sl_3arr(i)(j) <= bn_tx_sl_3arr(j)(i);
      END GENERATE;
    END GENERATE;
  END GENERATE;
  
  -- Actual UniBoard PCB mesh connect for transpose
  gen_reorder : IF g_reorder=TRUE GENERATE
                                        -- BN, phy  <= FN, phy
    bn_rx_sl_3arr(0)(0) <= fn_tx_sl_3arr(3)(1);  -- 0,0 <= 3,1
    bn_rx_sl_3arr(0)(1) <= fn_tx_sl_3arr(2)(0);  -- 0,1 <= 2,0
    bn_rx_sl_3arr(0)(2) <= fn_tx_sl_3arr(1)(0);  -- 0,2 <= 1,0
    bn_rx_sl_3arr(0)(3) <= fn_tx_sl_3arr(0)(0);  -- 0,3 <= 0,0
    
    bn_rx_sl_3arr(1)(0) <= fn_tx_sl_3arr(3)(0);  -- 1,0 <= 3,0
    bn_rx_sl_3arr(1)(1) <= fn_tx_sl_3arr(2)(1);  -- 1,1 <= 2,1
    bn_rx_sl_3arr(1)(2) <= fn_tx_sl_3arr(1)(1);  -- 1,2 <= 1,1
    bn_rx_sl_3arr(1)(3) <= fn_tx_sl_3arr(0)(3);  -- 1,3 <= 0,3
    
    bn_rx_sl_3arr(2)(0) <= fn_tx_sl_3arr(3)(2);  -- 2,0 <= 3,2
    bn_rx_sl_3arr(2)(1) <= fn_tx_sl_3arr(2)(2);  -- 2,1 <= 2,2
    bn_rx_sl_3arr(2)(2) <= fn_tx_sl_3arr(0)(2);  -- 2,2 <= 0,2
    bn_rx_sl_3arr(2)(3) <= fn_tx_sl_3arr(1)(3);  -- 2,3 <= 1,3
    
    bn_rx_sl_3arr(3)(0) <= fn_tx_sl_3arr(3)(3);  -- 3,0 <= 3,3
    bn_rx_sl_3arr(3)(1) <= fn_tx_sl_3arr(0)(1);  -- 3,1 <= 0,1
    bn_rx_sl_3arr(3)(2) <= fn_tx_sl_3arr(2)(3);  -- 3,2 <= 2,3
    bn_rx_sl_3arr(3)(3) <= fn_tx_sl_3arr(1)(2);  -- 3,3 <= 1,2
    
                                         -- FN, phy <= BN, phy
    fn_rx_sl_3arr(0)(0) <= bn_tx_sl_3arr(0)(3);  -- 0,0 <= 0,3
    fn_rx_sl_3arr(0)(1) <= bn_tx_sl_3arr(3)(1);  -- 0,1 <= 3,1
    fn_rx_sl_3arr(0)(2) <= bn_tx_sl_3arr(2)(2);  -- 0,2 <= 2,2
    fn_rx_sl_3arr(0)(3) <= bn_tx_sl_3arr(1)(3);  -- 0,3 <= 1,3
    
    fn_rx_sl_3arr(1)(0) <= bn_tx_sl_3arr(0)(2);  -- 1,0 <= 0,2
    fn_rx_sl_3arr(1)(1) <= bn_tx_sl_3arr(1)(2);  -- 1,1 <= 1,2
    fn_rx_sl_3arr(1)(2) <= bn_tx_sl_3arr(3)(3);  -- 1,2 <= 3,3
    fn_rx_sl_3arr(1)(3) <= bn_tx_sl_3arr(2)(3);  -- 1,3 <= 2,3
    
    fn_rx_sl_3arr(2)(0) <= bn_tx_sl_3arr(0)(1);  -- 2,0 <= 0,1
    fn_rx_sl_3arr(2)(1) <= bn_tx_sl_3arr(1)(1);  -- 2,1 <= 1,1
    fn_rx_sl_3arr(2)(2) <= bn_tx_sl_3arr(2)(1);  -- 2,2 <= 2,1
    fn_rx_sl_3arr(2)(3) <= bn_tx_sl_3arr(3)(2);  -- 2,3 <= 3,2
    
    fn_rx_sl_3arr(3)(0) <= bn_tx_sl_3arr(1)(0);  -- 3,0 <= 1,0
    fn_rx_sl_3arr(3)(1) <= bn_tx_sl_3arr(0)(0);  -- 3,1 <= 0,0
    fn_rx_sl_3arr(3)(2) <= bn_tx_sl_3arr(2)(0);  -- 3,2 <= 2,0
    fn_rx_sl_3arr(3)(3) <= bn_tx_sl_3arr(3)(0);  -- 3,3 <= 3,0
  END GENERATE;
  
END beh;
