--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

-- Purpose:
-- . Calculate the cross- and auto correlations of g_nof_inputs inputs.
-- Description:
-- . 

ENTITY correlator IS
  GENERIC (
    g_nof_inputs  : NATURAL;
    g_nof_mults   : NATURAL;
    g_data_w      : NATURAL := 16;
    g_conjugate   : BOOLEAN := TRUE
   ); 
  PORT (
    rst            : IN  STD_LOGIC;
    clk            : IN  STD_LOGIC;

    snk_in_arr     : IN  t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0);

    src_out_arr    : OUT t_dp_sosi_arr(g_nof_inputs*(g_nof_inputs+1)/2-1 DOWNTO 0)
  );
END correlator;

ARCHITECTURE str OF correlator IS

  SIGNAL corr_permutator_src_out_2arr_2 : t_dp_sosi_2arr_2(g_nof_inputs*(g_nof_inputs+1)/2-1 DOWNTO 0); -- Array of pairs
  SIGNAL corr_folder_src_out_2arr_2     : t_dp_sosi_2arr_2(g_nof_inputs*(g_nof_inputs+1)/2-1 DOWNTO 0); -- Array of pairs, not folded yet
  SIGNAL corr_multiplier_src_out_arr    : t_dp_sosi_arr(g_nof_mults-1 DOWNTO 0);
  SIGNAL corr_accumulator_src_out_arr   : t_dp_sosi_arr(g_nof_mults-1 DOWNTO 0);

BEGIN

  -----------------------------------------------------------------------------
  -- Create all unique pair permutations of the input streams
  -----------------------------------------------------------------------------
  u_corr_permutator : ENTITY work.corr_permutator
  GENERIC MAP (
    g_nof_inputs => g_nof_inputs
  )
  PORT MAP (
    clk            => clk,
    rst            => rst,

    snk_in_arr     => snk_in_arr,
    src_out_2arr_2 => corr_permutator_src_out_2arr_2
  );     

  -----------------------------------------------------------------------------
  -- Fold the streams if g_nof_mults < g_nof_inputs
  -- . Folder not implemented yet; section bypassed.
  -----------------------------------------------------------------------------
  corr_folder_src_out_2arr_2 <= corr_permutator_src_out_2arr_2;

  -----------------------------------------------------------------------------
  -- Complex multiplier stage
  -----------------------------------------------------------------------------
  u_corr_multiplier : ENTITY work.corr_multiplier
  GENERIC MAP (
    g_nof_inputs => g_nof_mults,
    g_data_w     => g_data_w,
    g_conjugate  => g_conjugate
  )
  PORT MAP (
    clk            => clk,
    rst            => rst,

    snk_in_2arr_2  => corr_folder_src_out_2arr_2,
    src_out_arr    => corr_multiplier_src_out_arr
  );  

  -----------------------------------------------------------------------------
  -- Accumulator stage
  -----------------------------------------------------------------------------
  u_corr_accumulator : ENTITY work.corr_accumulator
  GENERIC MAP (
    g_nof_inputs        => g_nof_mults,
    g_accumulator_depth => 0,
    g_data_w            => 64
  )
  PORT MAP (
    clk            => clk,
    rst            => rst,

    snk_in_arr     => corr_multiplier_src_out_arr,
    src_out_arr    => corr_accumulator_src_out_arr
  );

  src_out_arr <= corr_accumulator_src_out_arr;

END str;
