--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
--USE dp_lib.tb_dp_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;

ENTITY tb_corr_carousel IS
  GENERIC (
    g_nof_inputs  : NATURAL := 8;
    g_nof_outputs : NATURAL := 9
  );
END tb_corr_carousel;

ARCHITECTURE tb OF tb_corr_carousel IS

  CONSTANT c_clk_period                   : TIME := 10 ns;
  CONSTANT c_data_w                       : NATURAL := 32;
  CONSTANT c_nof_permutations             : NATURAL := (g_nof_inputs*(g_nof_inputs+1))/2;
  CONSTANT c_nof_clk_cycles_per_input_val : NATURAL := c_nof_permutations/g_nof_outputs;

  SIGNAL tb_end         : STD_LOGIC := '0';
  SIGNAL clk            : STD_LOGIC := '1';
  SIGNAL rst            : STD_LOGIC;

  SIGNAL carousel_snk_in_arr     : t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0);
  SIGNAL carousel_src_out_2arr_2 : t_dp_sosi_2arr_2(g_nof_outputs-1 DOWNTO 0);  

BEGIN

  clk <= NOT clk OR tb_end AFTER c_clk_period/2;
  rst <= '1', '0' AFTER c_clk_period*7;

  u_corr_carousel : ENTITY work.corr_carousel
  GENERIC MAP (
    g_nof_inputs  => g_nof_inputs,
    g_nof_outputs => g_nof_outputs
  )
  PORT MAP (
    rst         => rst,
    clk         => clk,
    snk_in_arr  => carousel_snk_in_arr,
    src_out_arr => carousel_src_out_2arr_2
  );  

  p_stimuli: PROCESS
  BEGIN
    WAIT UNTIL rst = '0';
    WAIT FOR 100 ns;
    FOR i IN 0 TO 10-1 LOOP
      FOR j IN 0 TO g_nof_inputs-1 LOOP
        -- Feed the carousel. Use the stream index as data word
        carousel_snk_in_arr(j).data(c_data_w-1 DOWNTO 0) <= TO_UVEC(j, c_data_w);
        carousel_snk_in_arr(j).valid <= '1';
      END LOOP;
      WAIT FOR c_clk_period;
      FOR j IN 0 TO g_nof_inputs-1 LOOP
        -- Let the carousel do its thing
        carousel_snk_in_arr(j).data <= (OTHERS=>'0');
        carousel_snk_in_arr(j).valid <= '0';
      END LOOP;
    WAIT FOR (c_nof_clk_cycles_per_input_val-1)*c_clk_period;
    END LOOP;
  END PROCESS;

END tb;
