--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, technology_lib, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;

ENTITY tech_10gbase_r IS
  GENERIC (
    g_technology      : NATURAL := c_tech_select_default;
    g_nof_phy         : NATURAL := 1
  );                      
  PORT (
    sys_clk                 : IN  STD_LOGIC;    -- 200 MHz
    sys_rst                 : IN  STD_LOGIC;

    -- Transceiver PLL reference clock
    tr_ref_clk              : IN  STD_LOGIC;    -- 644.531250 MHz

    tx_core_clk_in          : IN  STD_LOGIC;    -- 156.25 MHz
    rx_core_clk_in          : IN  STD_LOGIC;    -- 156.25 MHz
    
    tx_pma_div_clk_out      : OUT STD_LOGIC;
    tx_pma_clk_out          : OUT STD_LOGIC;
    tx_clk_out              : OUT STD_LOGIC;    -- 156.25 MHz
    rx_clk_out              : OUT STD_LOGIC;    -- 156.25 MHz
    
    xgmii_tx_dc_arr         : IN  t_xgmii_dc_arr(g_nof_phy-1 DOWNTO 0);
    xgmii_rx_dc_arr         : OUT t_xgmii_dc_arr(g_nof_phy-1 DOWNTO 0);

    tx_serial_arr           : OUT STD_LOGIC_VECTOR(g_nof_phy-1 DOWNTO 0);
    rx_serial_arr           : IN  STD_LOGIC_VECTOR(g_nof_phy-1 DOWNTO 0)
  );
END tech_10gbase_r;


ARCHITECTURE str OF tech_10gbase_r IS  

BEGIN
 
  gen_ip_arria10 : IF g_technology=c_tech_arria10 GENERATE
    u0 : ENTITY work.tech_10gbase_r_arria10
    GENERIC MAP (g_nof_phy)
    PORT MAP (sys_clk, sys_rst,
              tr_ref_clk, tx_core_clk_in, rx_core_clk_in,
              tx_pma_div_clk_out, tx_pma_clk_out, tx_clk_out, rx_clk_out,
              xgmii_tx_dc_arr(0), xgmii_rx_dc_arr(0),
              tx_serial_arr(0), rx_serial_arr(0));
  END GENERATE;
      
END str;

