import multiprocessing as mp
import time

class Port(object):
    def __init__(self, name, stream_index):
        self.name = name
        self.stream_index = stream_index
        self.pipe_end = None

    #'>' operator: Connect
    def __gt__(self, other):
        return self.__connect__(other)

    def __connect__(self, other):
        snk_end, src_end = mp.Pipe(duplex=False)
        self.pipe_end  = src_end
        other.pipe_end = snk_end
        return Connection(self.name, other.name)

    def send(self, item):
        self.pipe_end.send(item) 

    def recv(self):
        while True:
            try:
                yield self.pipe_end.recv()
            except EOFError:
                break

class Connection():
    def __init__(self, src_component, snk_component):
        self.vhdl_assignments = ['niets']

    def generate(self, file_name):
        target_file = open(file_name, "w")
        # Write component instance
        target_file.write(self.vhdl_assignments[0])
        target_file.close()


class Composite(object):
    def __init__(self, name, components=[], connections=[]):
        self.name = name
        self.file_name = self.name
        self.components = components
        self.connections = connections

    def start(self):
        for component in self.components:
            print 'Starting', component
            component.start()

    def terminate(self):
        for component in self.components:
            print 'Terminating', component
            component.terminate()

    def run_time(self, time_sec):
        self.start()
        time.sleep(time_sec)
        self.terminate()

    def generate(self):
        # write ASTRON header
        # []
        # write included libraries and packages
        # []
        # write entity declaration
        # []
        # write 'ARCHITECTURE str OF self.name IS'
        # []
        # write signal declarations
#        for component in self.components:
#            for port in component.signals:
#                signal.generate(self.file_name)

        # Write 'BEGIN'
        # []
        # write structural architecture (component instances)
        for component in self.components:
            component.generate(self.file_name)
        # write signal assignments
        for connection in self.connections:
            connection.generate(self.file_name)
        # write 'END str;'

class PortArray(list):
    def __init__(self, name, nof_streams):
        list.__init__(self)
        self.name = name
        for stream_index in range(nof_streams):
            self.append(Port(name, stream_index))

class Component(mp.Process):
    def __init__(self, name):
        mp.Process.__init__(self)
        self.name = name
        self.port_arrs = {}

    def set_port_arr(self, name, nof_streams):
        self.port_arrs[name] = PortArray(name, nof_streams)


    def generate(self, file_name):
        target_file = open(file_name, "w")
        # Write component instance
        target_file.write(self.vhdl_instance)
        target_file.close()


class Counter(Component):
    def __init__(self, name, nof_streams):
        Component.__init__(self, name)
        self.vhdl_instance = 'A string that contains the Counter VHDL instance'
        self.set_port_arr('src_out_arr', nof_streams)

    def run(self):
        for src in self.port_arrs['src_out_arr']:
            for item in xrange(10):
                print 'sending', item
                src.send(item) 

class Through(Component):
    def __init__(self, name, nof_streams):
        Component.__init__(self, name)
        self.vhdl_instance = 'A string that contains the Through VHDL instance'
        self.set_port_arr('src_out_arr', nof_streams)
        self.set_port_arr('snk_in_arr', nof_streams)
 
    def run(self):
        for snk,src in zip(self.port_arrs['snk_in_arr'], self.port_arrs['src_out_arr']):
            for item in snk.recv():
                print 'snk', snk, ': forwarding', item
                src.send(item) 
    
class Printer(Component):
    def __init__(self, name, nof_streams):
        Component.__init__(self, name)
        self.vhdl_instance = 'A string that contains the Printer VHDL instance'
        self.set_port_arr('snk_in_arr', nof_streams)

    def run(self):
        for snk in self.port_arrs['snk_in_arr']:
            for item in snk.recv():
                print 'snk', snk, ': receiving', item

a = Counter('counter', 1) # Passed name determines instance name and signal names
b = Through('through', 1)
c = Printer('printer', 1)

ab = a.port_arrs['src_out_arr'][0] > b.port_arrs['snk_in_arr'][0]
bc = b.port_arrs['src_out_arr'][0] > c.port_arrs['snk_in_arr'][0]


d = Composite('my_composite', [a,b,c], [ab,bc])

d.run_time(1)
d.generate()


