#!/bin/bash
###############################################################################
#
# Copyright (C) 2014
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

# Only run this script once
if [ -z "${UNB_SH}" ]; then
export UNB_SH=true
echo "Setup old UniBoard environment for Modelsim and Quartus"

# 1) Usage

# Source this script after the setup_radiohdl.sh in the .bashrc to setup the old UniBoard environment for Modelsim and Quartus.


# 2) Default tool settings

# The $RADIOHDL environment variable is defined by first calling setup_radiohdl.sh in the .bashrc.

. ${RADIOHDL}/tools/quartus/quartus_version.sh 11.1
. ${RADIOHDL}/tools/quartus/quartus_generic.sh
. ${RADIOHDL}/tools/modelsim/modelsim_version.sh 6.6c


# 3) Synthesis

# For Quartus synthesis in the old UniBoard environment no further settings are needed.


# 4) Simulation

# For Modelsim simulation in the old UniBoard environment the following setings are needed:

# The old UniBoard environment uses a libraries.ini that gets set in a project specific modelsim.ini. The default modelsim.ini
# gets overrulled via the MODELSIM environment variable. For RadioHDL the default modelsim.ini as it comes with the tool is
# used, because that is fine. This implies that a libraries.ini can not be invoked via the modelsim.ini. Therefore instead of
# using a central libraries.ini for RadioHDL all libraries get mapped directly in the generated modelsim project files.

# The $UNB environment variable is already defined in the setup_radiohdl.sh in the .bashrc.

# To simulate the old UniBoard $UNB environment do:
#
#   > unb_msim &
#

# The old UniBoard envrionment uses dedicated libraries.ini and commands.do for some application projects. These projects can
# be simulated by starting Modelsim with:
#
#   > paasar_msim &
#   > aaf_msim &
#
# In addation to the 'lp' and 'mk' commands for the UNB libraries these projects have 'lpa' and 'mka' commands to load and make
# the application specific libraries for simulation.

export AARTFAAC=${SVN}/Aartfaac/trunk
export PAASAR=${SVN}/Paasar/trunk

export PATH=${PATH}:\
${AARTFAAC}/Firmware/sim/modelsim:\
${PAASAR}/Firmware/sim/modelsim

# When Modelsim is started using the unb_msim script the setup sofar is sufficient.

# The alias unb_sim='vsim -do ${UNB}/Firmware/sim/modelsim/commands.do' is no longer supported, because the unb_msim script is
# equivalent in usage and easier to maintain because the start script contains the required settings. Or add to the .bashrc:
#
# alias unb_sim=unb_msim
#

fi
